--[[-------------------------------------------------------------------------
	DontSoloQueue
	Please contact Morsker through PM on wowace.com.

	Copyright (c) 2011  Morsker

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files (the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
	THE SOFTWARE.
---------------------------------------------------------------------------]]

local CreateFrame = CreateFrame
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers
local IsAltKeyDown = IsAltKeyDown
local PVPFrame = PVPFrame
local PVPFrameLeftButton = PVPFrameLeftButton

local frame
local altKeyWasDown

-- The central hook that's reached in several ways:
-- 1. Once when the addon loads.
-- 2. When PVPFrame changes its selected tab.
-- 3. On events that might involve joining / leaving party.
-- 4. On Alt key press / release.
--
-- Because this is generic, it doesn't make assumptions, and checks everything relevant.
-- It's called so rarely that optimization would be a mistake.
local function FixSoloQueueButton()
	if PVPFrame:IsShown() and PVPFrame.selectedTab == 1 then
		if IsAltKeyDown() or (GetNumPartyMembers() == 0 and GetNumRaidMembers() == 0) then
			PVPFrameLeftButton:Enable()
		else
			PVPFrameLeftButton:Disable()
		end
	end
end

-- Watch for Alt key press / release.
local function CheckAltOnUpdate()
	if altKeyWasDown ~= IsAltKeyDown() then
		altKeyWasDown = IsAltKeyDown()
		FixSoloQueueButton()
	end
end

-- Only register events when PVPFrame is shown.
local function PVPFrame_Shown()
	altKeyWasDown = IsAltKeyDown()
	if frame then
		frame:Show() -- start OnUpdate polling
	else
		frame = CreateFrame("frame")
		frame:SetScript("OnUpdate", CheckAltOnUpdate)
		frame:SetScript("OnEvent", FixSoloQueueButton)
	end
	frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
	frame:RegisterEvent("RAID_ROSTER_UPDATE")
end

-- Only register events when PVPFrame is shown.
local function PVPFrame_Hidden()
	frame:Hide() -- stop OnUpdate polling
	frame:UnregisterEvent("PARTY_MEMBERS_CHANGED")
	frame:UnregisterEvent("RAID_ROSTER_UPDATE")
end

hooksecurefunc("PVPFrame_TabClicked", FixSoloQueueButton)
FixSoloQueueButton()

PVPFrame:HookScript("OnShow", PVPFrame_Shown)
PVPFrame:HookScript("OnHide", PVPFrame_Hidden)
if PVPFrame:IsShown() then
	PVPFrame_Shown()
end
